import sys
import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_list_of_orders, assert_order, assert_list_of_payments


class TestOrder(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.order_id = 'dummy_order_id'
        self.order_url = payabbhi.api_base + '/api/v1/orders'

    @responses.activate
    def test_order_all(self):
        result = mock_file('dummy_order_collection')
        responses.add(responses.GET, self.order_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.order.all()
        resp = json.loads(result)
        assert_list_of_orders(self, response, resp)

    @responses.activate
    def test_order_with_options(self):
        result = mock_file('dummy_order_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.order_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.order.all(data={'count': count, 'skip':skip})
        resp = json.loads(result)
        assert_list_of_orders(self, response, resp)

    @responses.activate
    def test_order_retrieve(self):
        result = mock_file('dummy_order')
        url = '{0}/{1}'.format(self.order_url, self.order_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.order.retrieve(self.order_id)
        resp = json.loads(result)
        assert_order(self, response, resp)

    @responses.activate
    def test_order_create(self):
        result = mock_file('dummy_order')
        url = self.order_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.order.create(data={'amount':100, 'merchant_order_id':'M123', 'currency':'INR', 'payment_auto_capture':False, 'notes':{"merchant_order_id":"M123"}})
        resp = json.loads(result)
        assert_order(self, response, resp)

    @responses.activate
    def test_order_retrieve_payments(self):
        result = mock_file('dummy_order_payments')
        url = '{0}/{1}/payments'.format(self.order_url, self.order_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        self.client.order.id = self.order_id
        response = self.client.order.payments()
        resp = json.loads(result)
        assert_list_of_payments(self, response, resp)

    @responses.activate
    def test_empty_order_retrieve_payments(self):
        if sys.version_info[0] <= 2:
            with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
                self.client.order.payments()
            self.assertTrue('message: Object Id not set' in str(error.exception))
            self.assertTrue('Object Id not set' in error.exception.description)
            self.assertIsNone(error.exception.http_status)
            self.assertIsNone(error.exception.field)



from .api_resource import APIResource


class InvoiceItem(APIResource):

    def __init__(self, client=None):
        super(InvoiceItem, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Invoice Items
        Args:
            data : Dictionary having keys using which invoice item list will be filtered
                count:              Count of invoice items to be retrieved
                skip:               Number of invoice items to be skipped
                to:                 Invoice Item list till this timestamp will be retrieved
                from:               Invoice Item list from this timestamp will be retrieved
        Returns:
            List of Invoice Item objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Invoice Item from given data
        Args:
            data : Dictionary having keys using which invoice item has to be created
                customer_id:  The unique identifier of the Customer who will pay this invoice
                name: Name of the invoice item
                amount: Amount of the invoice item
                currency: Currency of the invoice item amoount
                invoice_id: The unique identifier of the invoice to which this invoice item to be added
                subscription_id: The unique identifier of the subscription to which this invoice item is to be added as an addon
                description: Description of the invoice item
                quantity: Quantity of the invoice item
                notes: key value pair as notes
        Returns:
            Invoice Item object containing data for created invoice item
        """
        return self._post(self.class_url(), data, **kwargs)

    def retrieve(self, invoice_item_id, **kwargs):
        """"
        Retrieve Invoice Item for given Id
        Args:
            invoice_item_id : Id for which Invoice Item object is to be retrieved
        Returns:
            Invoice Item object for given invoice item Id
        """
        return self._retrieve(invoice_item_id, **kwargs)

    def invoices(self, invoice_item_id, data=None, **kwargs):
        """"
        Retrieve Invoices where the invoice item with invoice_item_id is attached
        Returns:
            List of Invoice objects
        """
        if data is None:
            data = {}

        url = "{0}/invoices".format(self.instance_url(invoice_item_id))
        return self._get(url, data, **kwargs)

    def delete(self, invoice_item_id, **kwargs):
        """"
        Delete Invoice Item for given Id
        Args:
            invoice_item_id : Id for which Invoice Item object is to be deleted
        Returns:
            Invoice Item object corresponding to the invoice item Id after successful deletion
        """
        return self._delete(invoice_item_id, **kwargs)



from .api_resource import APIResource


class Customer(APIResource):

    def __init__(self, client=None):
        super(Customer, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Customers
        Args:
            data : Dictionary having keys using which customer list will be filtered
                count:              Count of customers to be retrieved
                skip:               Number of customers to be skipped
                to:                 Customer list till this timestamp will be retrieved
                from:               Customer list from this timestamp will be retrieved
        Returns:
            List of Customer objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Customer from given data
        Args:
            data : Dictionary having keys using which customer has to be created
                email: Email ID of the Customer.
                contact_no: Contact Number of the Customer.
                name: Name of the Customer.
                gstin: GSTIN of the Customer.
                billing_address: Billing Address is a key-value pair denoting the billing address of the customer. Valid keys are address_line1, address_line2, city, state, pin.
                shipping_address: Shipping Address is a key-value pair denoting the shipping address of the customer. Valid keys are address_line1, address_line2, city, state, pin.
                notes: key value pair as notes
        Returns:
            Customer object containing data for created customer
        """
        return self._post(self.class_url(), data, **kwargs)

    def edit(self, customer_id, data, **kwargs):
        """"
        Updates existing Customer from given data
        Args:
             customer_id: The identifier of the customer which needs to be updated.
             data : Dictionary having keys using which customer has to be created
                 email: Email ID of the Customer.
                 contact_no: Contact Number of the Customer.
                 name: Name of the Customer.
                 gstin: GSTIN of the Customer.
                 billing_address: Billing Address is a key-value pair denoting the billing address of the customer. Valid keys are address_line1, address_line2, city, state, pin.
                 shipping_address: Shipping Address is a key-value pair denoting the shipping address of the customer. Valid keys are address_line1, address_line2, city, state, pin.
                 notes: key value pair as notes
        Returns:
            Customer object containing data for updated customer
        """
        return self._put(self.instance_url(customer_id), data, **kwargs)

    def retrieve(self, customer_id, **kwargs):
        """"
        Retrieve Customer for given Id
        Args:
            customer_id : Id for which Customer object has to be retrieved
        Returns:
            Customer object for given customer Id
        """
        return self._retrieve(customer_id, **kwargs)



import json
import sys

import payabbhi
import responses
import unittest2

from .helpers import (assert_invoice, assert_list_of_invoice_items,
                      assert_list_of_invoices, assert_list_of_payments,
                      mock_file)


class TestInvoice(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(
            access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.invoice_id = 'dummy_invoice_id'
        self.invoice_url = payabbhi.api_base + '/api/v1/invoices'

    @responses.activate
    def test_invoice_all(self):
        result = mock_file('dummy_invoice_collection')
        responses.add(responses.GET, self.invoice_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.all()
        resp = json.loads(result)
        assert_list_of_invoices(self, response, resp)

    @responses.activate
    def test_invoice_with_options(self):
        result = mock_file('dummy_invoice_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.invoice_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.all(data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_invoices(self, response, resp)

    @responses.activate
    def test_invoice_retrieve(self):
        result = mock_file('dummy_invoice')
        url = '{0}/{1}'.format(self.invoice_url, self.invoice_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.retrieve(self.invoice_id)
        resp = json.loads(result)
        assert_invoice(self, response, resp)

    @responses.activate
    def test_invoice_create(self):
        result = mock_file('dummy_invoice')
        url = self.invoice_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.create(data={'customer_id': 'dummy_customer_id', 'invoice_no': '123123123123', 'due_date': 1549176945,
                                                    'currency': 'INR', 'description': 'TestInvoice', 'notes': {"mode": "test"}, 'line_items': [{"id": "dummy_item_id"}]})
        resp = json.loads(result)
        assert_invoice(self, response, resp)

    @responses.activate
    def test_invoice_void(self):
        result = mock_file('dummy_invoice_void')
        url = '{0}/{1}/void'.format(self.invoice_url, self.invoice_id)
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.void(self.invoice_id)
        resp = json.loads(result)
        assert_invoice(self, response, resp)

    @responses.activate
    def test_invoice_retrieve_lineitems(self):
        result = mock_file('dummy_invoice_lineitems')
        url = '{0}/{1}/line_items'.format(self.invoice_url, self.invoice_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.line_items(self.invoice_id)
        resp = json.loads(result)
        assert_list_of_invoice_items(self, response, resp)

    @responses.activate
    def test_invoice_retrieve_payments(self):
        result = mock_file('dummy_invoice_payments')
        url = '{0}/{1}/payments'.format(self.invoice_url, self.invoice_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoice.payments(self.invoice_id)
        resp = json.loads(result)
        assert_list_of_payments(self, response, resp)



from ..error import InvalidRequestError
from .api_resource import APIResource


class Order(APIResource):

    def __init__(self, client=None):
        super(Order, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Orders
        Args:
            data : Dictionary having keys using which order list will be filtered
                count:              Count of orders to be retrieved
                skip:               Number of orders to be skipped
                to:                 Order list till this timestamp will be retrieved
                from:               Order list from this timestamp will be retrieved
                authorized:         True/False if only the orders having an authorized payment will be retrieved
                merchant_order_id:  Orders with this merchant_order_id will be retrieved
        Returns:
            List of Order objects
        """
        if data is None:
            data = {}
        return super(Order, self)._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Order from given data
        Args:
            data : Dictionary having keys using which order has to be created
                amount:  Amount of Order
                currency: Currency used in Order
                merchant_order_id: Merchant side unique identifier for the order
                notes: key value pair as notes
                payment_auto_capture: True/False if payment should be auto captured or not
        Returns:
            Order object containing data for created order
        """
        return self._post(self.class_url(), data, **kwargs)

    def retrieve(self, order_id, **kwargs):
        """"
        Retrieve Order for given Id
        Args:
            order_id : Id for which Order object has to be retrieved
        Returns:
            Order object for given order Id
        """
        return self._retrieve(order_id, **kwargs)

    def payments(self, data=None, **kwargs):
        """"
        Retrieve Payments for the Order object
        Returns:
            List of Payment objects
        """
        if data is None:
            data = {}

        if not hasattr(self, 'id'):
            raise InvalidRequestError('Object Id not set')

        url = "{0}/payments".format(self.instance_url(self.id))
        return self._get(url, data, **kwargs)



from types import ModuleType

import requests

from . import resources, utility

def capitalize_camel_case(string):
    return "".join(map(str.capitalize, string.split('_')))

# Create a dict of resource classes
RESOURCE_CLASSES = {}

for resource_name, resource_module in resources.__dict__.items():
    if isinstance(resource_module, ModuleType) and capitalize_camel_case(resource_name) in resource_module.__dict__:
        RESOURCE_CLASSES[resource_name.replace("_","")] = resource_module.__dict__[capitalize_camel_case(resource_name)]

UTILITY_CLASSES = {}
for utility_name, utility_module in utility.__dict__.items():
    if isinstance(utility_module, ModuleType) and utility_name.capitalize() in utility_module.__dict__:
        UTILITY_CLASSES[utility_name] = utility_module.__dict__[utility_name.capitalize()]


class Client(object):

    VERSION = '1.0.3'

    def __init__(self, access_id="", secret_key=""):
        self.session = requests.Session()
        self.access_id = access_id
        self.secret_key = secret_key

        self.cert_path = False
        self.app_info = {}

        # intializes each resource
        # injecting this client object into the constructor
        for resource_class_name, resource_klass in RESOURCE_CLASSES.items():
            setattr(self, resource_class_name, resource_klass(self))

        for utility_class_name, utility_klass in UTILITY_CLASSES.items():
            setattr(self, utility_class_name, utility_klass(self))

    def set_app_info(self, app_name, app_version="", app_url=""):
        self.app_info = {
            'name': app_name,
            'version': app_version,
            'url': app_url,
        }

    def get_app_info(self):
        return self.app_info



import sys
import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_list_of_events, assert_event


class TestEvent(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.event_id = 'dummy_event_id'
        self.event_url = payabbhi.api_base + '/api/v1/events'

    @responses.activate
    def test_event_all(self):
        result = mock_file('dummy_event_collection')
        responses.add(responses.GET, self.event_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.event.all()
        resp = json.loads(result)
        assert_list_of_events(self, response, resp)

    @responses.activate
    def test_event_with_options(self):
        result = mock_file('dummy_event_collection')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.event_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.event.all(data={'count': count, 'skip':skip})
        resp = json.loads(result)
        assert_list_of_events(self, response, resp)

    @responses.activate
    def test_event_retrieve(self):
        result = mock_file('dummy_event')
        url = '{0}/{1}'.format(self.event_url, self.event_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.event.retrieve(self.event_id)
        resp = json.loads(result)
        assert_event(self, response, resp)



from .api_resource import APIResource


class Plan(APIResource):

    def __init__(self, client=None):
        super(Plan, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Plans
        Args:
            data : Dictionary having keys using which plan list will be filtered
                count:              Count of plans to be retrieved
                skip:               Number of plans to be skipped
                to:                 Plan list till this timestamp will be retrieved
                from:               Plan list from this timestamp will be retrieved
                product_id:         A filter on the plan list based on the product_id field.
        Returns:
            List of Plan objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Plan from given data
        Args:
            data : Dictionary having keys using which plan has to be created
                product_id: The identifier of the Product against which plan was created.
                amount: A positive integer in the smallest currency unit (e.g., 5000 paisa denotes Rs 50.00). Minimum amount is 100 paise.
                currency: Three-letter ISO currency code. Currently only INR is supported.
                frequency: Frequency of billing interval.
                interval: Interval at which the Subscription would be billed. This can be day(s), week(s), month(s) or year(s).
                name: Name of the Plan. Ex: Basic, Standard etc.
                notes: key value pair as notes
        Returns:
            Plan object containing data for created plan
        """
        return self._post(self.class_url(), data, **kwargs)

    def retrieve(self, plan_id, **kwargs):
        """"
        Retrieve Plan for given Id
        Args:
            plan_id : Id for which Plan object has to be retrieved
        Returns:
            Plan object for given plan Id
        """
        return self._retrieve(plan_id, **kwargs)



import sys
import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_list_of_transfers, assert_transfer


class TestTransfer(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.transfer_id = 'dummy_transfer_id'
        self.transfer_url = payabbhi.api_base + '/api/v1/transfers'
        self.source_id = 'dummy_source_id'

    @responses.activate
    def test_transfer_all(self):
        result = mock_file('dummy_transfer_collection')
        responses.add(responses.GET, self.transfer_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.transfer.all()
        resp = json.loads(result)
        assert_list_of_transfers(self, response, resp)

    @responses.activate
    def test_transfer_with_options(self):
        result = mock_file('dummy_transfer_collection')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.transfer_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.transfer.all(data={'count': count, 'skip':skip})
        resp = json.loads(result)
        assert_list_of_transfers(self, response, resp)

    @responses.activate
    def test_transfer_retrieve(self):
        result = mock_file('dummy_transfer')
        url = '{0}/{1}'.format(self.transfer_url, self.transfer_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.transfer.retrieve(self.transfer_id)
        resp = json.loads(result)
        assert_transfer(self, response, resp)

    @responses.activate
    def test_transfer_create(self):
        result = mock_file('dummy_transfer_collection')
        url = payabbhi.api_base + '/api/v1/payments/' + self.source_id + '/transfers'
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.transfer.create(self.source_id, data={'transfers':[{'amount':20,'currency':'INR','recipient_id':'recp_Za30i2k3p6blq3i1'},
                                                                                  {'amount':30,'currency':'INR','recipient_id':'recp_Y2ojRlJVqRMhB0Ay'}]})
        resp = json.loads(result)
        assert_list_of_transfers(self, response, resp)



from .api_resource import APIResource


class Refund(APIResource):

    def __init__(self, client=None):
        super(Refund, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Refunds
        Args:
            data : Dictionary having keys using which order list will be filtered
                count:           Count of refunds to be retrieved
                skip:            Number of refunds to be skipped
                to:              Refund list till this timestamp will be retrieved
                from:            Refund list from this timestamp will be retrieved
        Returns:
            List of Refund objects
        """
        if data is None:
            data = {}

        return super(Refund, self)._all(data, **kwargs)

    def retrieve(self, refund_id, **kwargs):
        """"
        Retrieve Refund for given Id
        Args:
            refund_id : Id for which refund object has to be retrieved
        Returns:
            Refund object with given refund Id
        """
        return super(Refund, self)._retrieve(refund_id, **kwargs)

    def create(self, payment_id, data=None, **kwargs):
        """"
        Create Refund from given data and payment Id
        Args:
            payment_id : Id for which Refund has to be created
            data : Dictionary having keys using which order have to be created
                'amount' :  Amount to be refunded
                'notes' : key value pair as notes
        Returns:
            Refund object containing data for created refund
        """
        if data is None:
            data = {}

        url = '/api/v1/payments/' + payment_id + '/refunds'
        return self._post(url, data, **kwargs)



import sys
import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_payment, assert_list_of_refunds, assert_refund, assert_list_of_payments, assert_list_of_transfers

class TestPayment(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.payment_id = 'dummy_payment_id'
        self.payment_url = payabbhi.api_base + '/api/v1/payments'

    @responses.activate
    def test_payment_all(self):
        result = mock_file('dummy_payment_collection')
        responses.add(responses.GET, self.payment_url, status=200,
                      body=result, match_querystring=True, content_type="application/json")
        response = self.client.payment.all()
        resp = json.loads(result)
        assert_list_of_payments(self, response, resp)

    @responses.activate
    def test_payment_with_options(self):
        result = mock_file('dummy_payment_collection_filters')
        count = 4
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.payment_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.all(data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_payments(self, response, resp)

    @responses.activate
    def test_payment_retrieve(self):
        result = mock_file('dummy_payment')
        url = '{0}/{1}'.format(self.payment_url, self.payment_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.retrieve(self.payment_id)
        resp = json.loads(result)
        assert_payment(self, response, resp)

    @responses.activate
    def test_payment_retrieve_refunds(self):
        result = mock_file('dummy_payment_refunds')
        url = '{0}/{1}/refunds'.format(self.payment_url, self.payment_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.refunds(self.payment_id)
        resp = json.loads(result)
        assert_list_of_refunds(self, response, resp)

    @responses.activate
    def test_payment_retrieves_refunds_with_options(self):
        result = mock_file('dummy_payment_refunds_collection_filters')
        count = 1
        url = '{0}/{1}/refunds?count={2}'.format(self.payment_url, self.payment_id, count)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.refunds(self.payment_id, data={'count':1})
        resp = json.loads(result)
        assert_list_of_refunds(self, response, resp)

    @responses.activate
    def test_payment_capture(self):
        result = mock_file('dummy_payment_capture')
        url = '{0}/{1}/capture'.format(self.payment_url, self.payment_id)
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        self.client.payment.id = self.payment_id
        response = self.client.payment.capture()
        resp = json.loads(result)
        assert_payment(self, response, resp)

    @responses.activate
    def test_empty_payment_capture(self):
        if sys.version_info[0] <= 2:
            with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
                self.client.payment.capture()
            self.assertTrue('message: Object Id not set' in str(error.exception))
            self.assertTrue('Object Id not set' in error.exception.description)
            self.assertIsNone(error.exception.http_status)
            self.assertIsNone(error.exception.field)

    @responses.activate
    def test_payment_refund(self):
        refund_result = mock_file('dummy_payment_refund')
        refund_url = '{0}/{1}/refunds'.format(self.payment_url, self.payment_id)
        responses.add(responses.POST, refund_url, status=200,
                      body=refund_result, match_querystring=True)

        payment_result = mock_file('dummy_payment')
        retrieve_payment_url = '{0}/{1}'.format(self.payment_url, self.payment_id)
        responses.add(responses.GET, retrieve_payment_url, status=200, body=payment_result, match_querystring=True)

        self.client.payment.id = self.payment_id

        refund_response = self.client.payment.refund(data={'amount':5000, 'currency':'INR', 'notes':{'reason': 'order cancelled'}})
        resp = json.loads(refund_result)
        assert_refund(self, refund_response, resp)

        payment_response = self.client.payment.retrieve(self.payment_id)
        payment_resp = json.loads(payment_result)
        assert_payment(self, payment_response, payment_resp)

    @responses.activate
    def test_empty_payment_refund(self):
        if sys.version_info[0] <= 2:
            with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
                self.client.payment.refund()
            self.assertTrue('message: Object Id not set' in str(error.exception))
            self.assertTrue('Object Id not set' in error.exception.description)
            self.assertIsNone(error.exception.http_status)
            self.assertIsNone(error.exception.field)

    @responses.activate
    def test_payment_retrieve_transfers(self):
        result = mock_file('dummy_transfer_collection')
        url = '{0}/{1}/transfers'.format(self.payment_url, self.payment_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.transfers(self.payment_id)
        resp = json.loads(result)
        assert_list_of_transfers(self, response, resp)

    @responses.activate
    def test_payment_retrieves_transfers_with_options(self):
        result = mock_file('dummy_transfer_collection')
        count = 1
        url = '{0}/{1}/transfers?count={2}'.format(self.payment_url, self.payment_id, count)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.transfers(self.payment_id, data={'count':1})
        resp = json.loads(result)
        assert_list_of_transfers(self, response, resp)



import os
import payabbhi
import unittest2


def mock_file(filename):
    if not filename:
        return ''
    file_dir = os.path.dirname(__file__)
    file_path = "{0}/mocks/{1}.json".format(file_dir, filename)
    with open(file_path, 'r') as f:
        return f.read()

def assert_order(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Order), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.status, expected.get('status'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.merchant_order_id, expected.get('merchant_order_id'))
    self.assertEqual(actual.payment_attempts, expected.get('payment_attempts'))


def assert_payment(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Payment), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.status, expected.get('status'))
    self.assertEqual(actual.description, expected.get('description'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.bank, expected.get('bank'))
    self.assertEqual(actual.card, expected.get('card'))
    self.assertEqual(actual.contact, expected.get('contact'))
    self.assertEqual(actual.email, expected.get('email'))
    self.assertEqual(actual.error_code, expected.get('error_code'))
    self.assertEqual(actual.error_description, expected.get('error_description'))
    self.assertEqual(actual.fee, expected.get('fee'))
    self.assertEqual(actual.international, expected.get('international'))
    self.assertEqual(actual.method, expected.get('method'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.order_id, expected.get('order_id'))
    self.assertEqual(actual.payout_amount, expected.get('payout_amount'))
    self.assertEqual(actual.payout_type, expected.get('payout_type'))
    self.assertEqual(actual.refund_status, expected.get('refund_status'))
    self.assertEqual(actual.refunded_amount, expected.get('refunded_amount'))
    self.assertEqual(actual.service_tax, expected.get('service_tax'))
    self.assertEqual(actual.wallet, expected.get('wallet'))
    assert_list_of_refunds(self, actual.refunds, expected.get('refunds'))

def assert_refund(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Refund), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.payment_id, expected.get('payment_id'))

def assert_product(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Product), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.name, expected.get('name'))
    self.assertEqual(actual.type, expected.get('type'))
    self.assertEqual(actual.unit_label, expected.get('unit_label'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.created_at, expected.get('created_at'))

def assert_plan(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Plan), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.name, expected.get('name'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.frequency, expected.get('frequency'))
    self.assertEqual(actual.interval, expected.get('interval'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.created_at, expected.get('created_at'))

def assert_customer(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Customer), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.name, expected.get('name'))
    self.assertEqual(actual.email, expected.get('email'))
    self.assertEqual(actual.contact_no, expected.get('contact_no'))
    self.assertEqual(actual.billing_address, expected.get('billing_address'))
    self.assertEqual(actual.shipping_address, expected.get('shipping_address'))
    self.assertEqual(actual.gstin, expected.get('gstin'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    assert_list_of_subscriptions(self, actual.subscriptions, expected.get('subscriptions'))

def assert_subscription(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Subscription), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.plan_id, expected.get('plan_id'))
    self.assertEqual(actual.customer_id, expected.get('customer_id'))
    self.assertEqual(actual.billing_method, expected.get('billing_method'))
    self.assertEqual(actual.quantity, expected.get('quantity'))
    self.assertEqual(actual.customer_notification_by, expected.get('customer_notification_by'))
    self.assertEqual(actual.billing_cycle_count, expected.get('billing_cycle_count'))
    self.assertEqual(actual.paid_count, expected.get('paid_count'))
    self.assertEqual(actual.cancel_at_period_end, expected.get('cancel_at_period_end'))
    self.assertEqual(actual.due_at, expected.get('due_at'))
    self.assertEqual(actual.trial_end_at, expected.get('trial_end_at'))
    self.assertEqual(actual.trial_duration, expected.get('trial_duration'))
    self.assertEqual(actual.status, expected.get('status'))
    self.assertEqual(actual.current_start_at, expected.get('current_start_at'))
    self.assertEqual(actual.current_end_at, expected.get('current_end_at'))
    self.assertEqual(actual.ended_at, expected.get('ended_at'))
    self.assertEqual(actual.cancelled_at, expected.get('cancelled_at'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.created_at, expected.get('created_at'))

def assert_invoice_item(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.InvoiceItem), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.name, expected.get('name'))
    self.assertEqual(actual.description, expected.get('description'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.customer_id, expected.get('customer_id'))
    self.assertEqual(actual.quantity, expected.get('quantity'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.deleted_at, expected.get('deleted_at'))

def assert_invoice(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Invoice), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.billing_method, expected.get('billing_method'))
    self.assertEqual(actual.customer_id, expected.get('customer_id'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.description, expected.get('description'))
    self.assertEqual(actual.due_date, expected.get('due_date'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.notify_by, expected.get('notify_by'))
    self.assertEqual(actual.payment_attempt, expected.get('payment_attempt'))
    self.assertEqual(actual.invoice_no, expected.get('invoice_no'))
    self.assertEqual(actual.status, expected.get('status'))
    self.assertEqual(actual.subscription_id, expected.get('subscription_id'))
    self.assertEqual(actual.url, expected.get('url'))
    assert_list_of_invoice_items(self, actual.line_items, expected.get('line_items'))

def assert_event(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Event), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.type, expected.get('type'))
    self.assertEqual(actual.created_at, expected.get('created_at'))
    self.assertEqual(actual.environment, expected.get('environment'))
    self.assertEqual(actual.data, expected.get('data'))

def assert_transfer(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.Transfer), True)
    self.assertEqual(actual.id, expected.get('id'))
    self.assertEqual(actual.object, expected.get('object'))
    self.assertEqual(actual.description, expected.get('description'))
    self.assertEqual(actual.source_id, expected.get('source_id'))
    self.assertEqual(actual.recipient_id, expected.get('recipient_id'))
    self.assertEqual(actual.amount, expected.get('amount'))
    self.assertEqual(actual.currency, expected.get('currency'))
    self.assertEqual(actual.fees, expected.get('fees'))
    self.assertEqual(actual.gst, expected.get('gst'))
    self.assertEqual(actual.notes, expected.get('notes'))
    self.assertEqual(actual.created_at, expected.get('created_at'))


def assert_list_of_orders(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (order_actual, order_expected) in zip(actual.data, expected.get('data')):
        assert_order(self, order_actual, order_expected)

def assert_list_of_payments(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (payment_actual, payment_expected) in zip(actual.data, expected.get('data')):
        assert_payment(self, payment_actual, payment_expected)

def assert_list_of_refunds(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (refund_actual, refund_expected) in zip(actual.data, expected.get('data')):
        assert_refund(self, refund_actual, refund_expected)

def assert_list_of_products(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (product_actual, product_expected) in zip(actual.data, expected.get('data')):
        assert_product(self, product_actual, product_expected)

def assert_list_of_plans(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (plan_actual, plan_expected) in zip(actual.data, expected.get('data')):
        assert_plan(self, plan_actual, plan_expected)

def assert_list_of_customers(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (customer_actual, customer_expected) in zip(actual.data, expected.get('data')):
        assert_customer(self, customer_actual, customer_expected)

def assert_list_of_subscriptions(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (subscription_actual, subscription_expected) in zip(actual.data, expected.get('data')):
        assert_subscription(self, subscription_actual, subscription_expected)

def assert_list_of_invoice_items(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (invoice_item_actual, invoice_item_expected) in zip(actual.data, expected.get('data')):
        assert_invoice_item(self, invoice_item_actual, invoice_item_expected)

def assert_list_of_invoices(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (invoice_actual, invoice_expected) in zip(actual.data, expected.get('data')):
        assert_invoice(self, invoice_actual, invoice_expected)

def assert_list_of_events(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (event_actual, event_expected) in zip(actual.data, expected.get('data')):
        assert_event(self, event_actual, event_expected)

def assert_list_of_transfers(self, actual, expected):
    self.assertEqual(isinstance(actual, payabbhi.resources.List), True)
    self.assertEqual(actual.total_count, expected.get('total_count'))
    self.assertEqual(actual.object, expected.get('object'))
    for (transfer_actual, transfer_expected) in zip(actual.data, expected.get('data')):
        assert_transfer(self, transfer_actual, transfer_expected)



import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_list_of_refunds, assert_refund

class TestRefund(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.refund_id = 'dummy_refund_id'
        self.refund_url = payabbhi.api_base + '/api/v1/refunds'
        self.payment_url = payabbhi.api_base + '/api/v1/payments'
        self.payment_id = 'dummy_payment_id'

    @responses.activate
    def test_refund_all(self):
        result = mock_file('dummy_refund_collection')
        responses.add(responses.GET, self.refund_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.refund.all()
        resp = json.loads(result)
        assert_list_of_refunds(self, response, resp)

    @responses.activate
    def test_refund_with_options(self):
        result = mock_file('dummy_refund_collection_filters')
        count = 5
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.refund_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.refund.all(data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_refunds(self, response, resp)

    @responses.activate
    def test_refund_retrieve(self):
        result = mock_file('dummy_refund')
        url = '{0}/{1}'.format(self.refund_url, self.refund_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.refund.retrieve(self.refund_id)
        resp = json.loads(result)
        assert_refund(self, response, resp)

    @responses.activate
    def test_refund_create(self):
        result = mock_file('dummy_payment_refund')
        url = '{0}/{1}/refunds'.format(self.payment_url, self.payment_id)
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.refund.create(self.payment_id, amount=5000, currency="INR", notes="{\"merchant_order_id\":\"M1234\"}")
        resp = json.loads(result)
        assert_refund(self, response, resp)



import responses
import time
import hmac
import hashlib
import payabbhi

import unittest2
from ..error import SignatureVerificationError

class TestClientValidator(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')

    @responses.activate
    def test_verify_payment_signature(self):
        sig = 'e70360e32919311d31cbc9b558ea9024715b816ce64293ffc992459a94daac42'
        parameters = {}
        parameters['order_id'] = 'dummy_order_id'
        parameters['payment_id'] = 'dummy_payment_id'
        parameters['payment_signature'] = sig

        self.assertEqual(self.client.utility.verify_payment_signature(parameters), True)

    @responses.activate
    def test_verify_payment_signature_with_exception(self):
        parameters = {}
        parameters['order_id'] = 'dummy_order_id'
        parameters['payment_id'] = 'dummy_payment_id'
        parameters['payment_signature'] = 'wrong_signature'

        self.assertRaises(SignatureVerificationError, self.client.utility.verify_payment_signature, parameters)

    @responses.activate
    def test_verify_webhook_signature(self):
        payload = '{"event":"payment.captured"}'
        t = int(time.time())
        secret="skw_live_jHNxKsDqJusco5hA"
        message = payload + '&' + str(t)
        v1 = hmac.new(secret, message, hashlib.sha256).hexdigest()
        actual_signature = "t=" + str(t) + ", v1=" + v1
        self.assertEqual(self.client.utility.verify_webhook_signature(payload,actual_signature,secret), True)

    @responses.activate
    def test_verify_webhook_signature_with_replay(self):
        payload = '{"event":"payment.captured"}'
        t = int(time.time())
        secret="skw_live_jHNxKsDqJusco5hA"
        message = payload + '&' + str(t)
        v1 = hmac.new(secret, message, hashlib.sha256).hexdigest()
        actual_signature = "t=" + str(t) + ", v1=" + v1
        self.assertEqual(self.client.utility.verify_webhook_signature(payload,actual_signature,secret,30), True)

    @responses.activate
    def test_verify_webhook_signature_with_error(self):
        payload = '{"event":"payment.captured"}'
        t = int(time.time()) - 40
        secret="skw_live_jHNxKsDqJusco5hA"
        message = payload + '&' + str(t)
        v1 = hmac.new(secret, message, hashlib.sha256).hexdigest()
        actual_signature = "t=" + str(t) + ", v1=" + v1
        self.assertRaises(SignatureVerificationError, self.client.utility.verify_webhook_signature, payload,actual_signature,secret,30)



import json
import sys

import payabbhi
import responses
import unittest2

from .helpers import (assert_invoice_item, assert_list_of_invoice_items,
                      assert_list_of_invoices, mock_file)


class TestInvoiceItem(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(
            access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.invoice_item_id = 'dummy_invoice_item_id'
        self.invoice_item_url = payabbhi.api_base + '/api/v1/invoiceitems'

    @responses.activate
    def test_invoice_item_all(self):
        result = mock_file('dummy_invoice_item_collection')
        responses.add(responses.GET, self.invoice_item_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.all()
        resp = json.loads(result)
        assert_list_of_invoice_items(self, response, resp)

    @responses.activate
    def test_invoice_item_with_options(self):
        result = mock_file('dummy_invoice_item_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(
            self.invoice_item_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.all(
            data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_invoice_items(self, response, resp)

    @responses.activate
    def test_invoice_item_retrieve(self):
        result = mock_file('dummy_invoice_item')
        url = '{0}/{1}'.format(self.invoice_item_url, self.invoice_item_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.retrieve(self.invoice_item_id)
        resp = json.loads(result)
        assert_invoice_item(self, response, resp)

    @responses.activate
    def test_invoice_item_create(self):
        result = mock_file('dummy_invoice_item')
        url = self.invoice_item_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.create(
            data={'customer_id': 'dummy_customer_id', 'name': 'Line Item', 'currency': 'INR', 'amount': 200})
        resp = json.loads(result)
        assert_invoice_item(self, response, resp)

    @responses.activate
    def test_invoice_item_retrieve_invoices(self):
        result = mock_file('dummy_invoice_collection')
        url = '{0}/{1}/invoices'.format(self.invoice_item_url,
                                        self.invoice_item_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.invoices(self.invoice_item_id)
        resp = json.loads(result)
        assert_list_of_invoices(self, response, resp)

    @responses.activate
    def test_invoice_item_delete(self):
        result = mock_file('dummy_invoice_item_delete')
        url = '{0}/{1}'.format(self.invoice_item_url, self.invoice_item_id)
        responses.add(responses.DELETE, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.invoiceitem.delete(self.invoice_item_id)
        resp = json.loads(result)
        assert_invoice_item(self, response, resp)



from .api_resource import APIResource


class Event(APIResource):

    def __init__(self, client=None):
        super(Event, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Events
        Args:
            data : Dictionary having keys using which event list will be filtered
                count:              Count of events to be retrieved
                skip:               Number of events to be skipped
                to:                 Event list till this timestamp will be retrieved
                from:               Event list from this timestamp will be retrieved
                type:               Types of events to be listed
        Returns:
            List of Event objects
        """
        if data is None:
            data = {}
        return super(Event, self)._all(data, **kwargs)

    def retrieve(self, event_id, **kwargs):
        """"
        Retrieve Event for given Id
        Args:
            event_id : Id for which Event object has to be retrieved
        Returns:
            Event object for given event Id
        """
        return self._retrieve(event_id, **kwargs)



from .api_resource import APIResource


class Product(APIResource):

    def __init__(self, client=None):
        super(Product, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Products
        Args:
            data : Dictionary having keys using which product list will be filtered
                count:              Count of products to be retrieved
                skip:               Number of products to be skipped
                to:                 Product list till this timestamp will be retrieved
                from:               Product list from this timestamp will be retrieved
        Returns:
            List of Product objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Product from given data
        Args:
            data : Dictionary having keys using which product has to be created
                name: Name of the Product, which should be displayed to the customer in receipt and invoice.
                type: Type of the Product. This can be good or service
                unit_label: Unit label describing the unit of the product being charged for metered billing - MB, text messages etc
                notes: key value pair as notes
        Returns:
            Product object containing data for created product
        """
        return self._post(self.class_url(), data, **kwargs)

    def retrieve(self, product_id, **kwargs):
        """"
        Retrieve Product for given Id
        Args:
            product_id : Id for which Product object has to be retrieved
        Returns:
            Product object for given product Id
        """
        return self._retrieve(product_id, **kwargs)



import hashlib
import hmac
import sys
import time

from ..error import SignatureVerificationError


class Utility(object):
    def __init__(self, client=None):
        self.client = client

    def verify_payment_signature(self, parameters):
        order_id = str(parameters['order_id'])
        payment_id = str(parameters['payment_id'])
        payment_signature = str(parameters['payment_signature'])

        msg = "{0}&{1}".format(payment_id, order_id)

        return self.verify_signature(payment_signature, msg, str(self.client.secret_key))


    def verify_webhook_signature(self, parameters, actual_signature, secret, replay_interval=300):
        entities = actual_signature.split(",")
        payload_map = {}
        for entity in entities:
            key_value = entity.split("=")
            payload_map[key_value[0].strip()] = key_value[1]

        if payload_map.get("t") == None or payload_map.get("v1") == None or int(time.time()) - int(payload_map["t"]) > replay_interval:
            raise SignatureVerificationError('Invalid signature passed')

        canonical_string  = "{0}&{1}".format(parameters, payload_map["t"])
        return self.verify_signature(payload_map["v1"],canonical_string, str(secret))


    def verify_signature(self, signature, body, key):

        if sys.version_info[0] == 3:  # pragma: no cover
            key = bytes(key, 'utf-8')
            body = bytes(body, 'utf-8')

        dig = hmac.new(key=key,
                       msg=body,
                       digestmod=hashlib.sha256)

        generated_signature = dig.hexdigest()

        if not generated_signature == signature:
            raise SignatureVerificationError('Invalid signature passed')

        return True



import inspect
import json
import sys

from payabbhi import http_client

try:
    import urlparse
    import urllib
except ImportError:
    import urllib.parse as urlparse


class APIResource(http_client.HTTPClient):

    def __init__(self, client=None):
        self.client = client

    def pretty_print(self):
        new_dict = {}
        for key, value in self.__dict__.items():
            if key != 'client':
                if isinstance(value, list):
                    new_dict[key] = [json.loads(i.pretty_print()) for i in value]
                elif isinstance(value, APIResource):
                    new_dict[key] = json.loads(value.pretty_print())
                else:
                    new_dict[key] = value
        return json.dumps(new_dict, indent=4)

    def __str__(self):
        return self.pretty_print()

    def __repr__(self):
        return self.pretty_print()

    def class_url(self):
        stack = inspect.stack()
        called_class = stack[1][0].f_locals["self"].__class__
        called_class = str(called_class)
        class_name = called_class[8:len(called_class)-2]
        splitted_class_name = str(class_name).split(".")
        cls_name = splitted_class_name[len(splitted_class_name)-1].lower()
        return "/api/v1/%ss" % (cls_name,)

    def instance_url(self, instance_id):
        class_url = self.class_url()
        if sys.version_info[0] >= 3:
            return class_url + '/' + urlparse.quote(instance_id.encode('utf8'))

        return class_url + '/' + urllib.quote(instance_id.encode('utf8'))

    def _all(self, data, **kwargs):
        return self._get(self.class_url(), data, **kwargs)

    def _retrieve(self, _id, **kwargs):
        return self._get(self.instance_url(_id), {}, **kwargs)

    def _get(self, path, params, **kwargs):
        return self.request('GET', path, self.client, params=params, **kwargs)

    def _post(self, path, data, **kwargs):
        return self.request('POST', path, self.client, data=data, **kwargs)

    def _put(self, path, data, **kwargs):
        return self.request('PUT', path, self.client, data=data, **kwargs)

    def _delete(self, _id, **kwargs):
        return self.request('DELETE', self.instance_url(_id), self.client, data=None, **kwargs)



from .resources import Payment
from .resources import Refund
from .resources import Order
from .resources import Product
from .resources import Plan
from .resources import Customer
from .resources import Subscription
from .resources import Invoice
from .resources import InvoiceItem
from .resources import List
from .resources import Event
from .resources import Transfer

from .utility import Utility

from .http_client import HTTPClient
from .client import Client
from .error import (
    APIConnectionError,
    APIError,
    AuthenticationError,
    InvalidRequestError,
    PayabbhiError,
    GatewayError)

api_base = 'https://payabbhi.com'


__all__ = [
    'Payment',
    'Refund',
    'Order',
    'Product',
    'Plan',
    'Customer',
    'Subscription',
    'Invoice',
    'InvoiceItem',
    'Event',
    'Transfer',
    'List',
    'Utility'
]



import responses
import payabbhi
import unittest2
from .helpers import mock_file


class TestAPIResource(unittest2.TestCase):

    def setUp(self):
        payabbhi.api_base = 'https://payabbhi.com'
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        self.client.set_app_info('TestApplication', 'beta', 'http://www.test.com')

    @responses.activate
    def test_class_url(self):
        self.assertEqual('/api/v1/testapiresources', payabbhi.resources.APIResource().class_url())

    @responses.activate
    def test_pretty_print(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_payment_collection')
        responses.add(responses.GET, payment_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.payment.all()
        response.pretty_print()



import sys
import json
import responses
import payabbhi

import unittest2
from .helpers import mock_file, assert_list_of_products, assert_product


class TestProduct(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.product_id = 'dummy_product_id'
        self.product_url = payabbhi.api_base + '/api/v1/products'

    @responses.activate
    def test_product_all(self):
        result = mock_file('dummy_product_collection')
        responses.add(responses.GET, self.product_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.product.all()
        resp = json.loads(result)
        assert_list_of_products(self, response, resp)

    @responses.activate
    def test_product_with_options(self):
        result = mock_file('dummy_product_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.product_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.product.all(data={'count': count, 'skip':skip})
        resp = json.loads(result)
        assert_list_of_products(self, response, resp)

    @responses.activate
    def test_product_retrieve(self):
        result = mock_file('dummy_product')
        url = '{0}/{1}'.format(self.product_url, self.product_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.product.retrieve(self.product_id)
        resp = json.loads(result)
        assert_product(self, response, resp)

    @responses.activate
    def test_product_create(self):
        result = mock_file('dummy_product')
        url = self.product_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.product.create(data={'name':'Books', 'unit_label':'MB', 'notes':{"genre":"comedy"}})
        resp = json.loads(result)
        assert_product(self, response, resp)



import sys
import responses
import payabbhi

import unittest2
from .helpers import mock_file

class TestHTTPClient(unittest2.TestCase):

    def setUp(self):
        payabbhi.api_base = 'https://payabbhi.com'
        self.client = payabbhi.Client(access_id='access_id', secret_key='secret_key')
        self.client.set_app_info('TestApplication', 'beta', 'http://www.test.com')

    def test_build_get_query(self):
        self.assertEqual(("https://payabbhi.com/hello/world", "/hello/world"), payabbhi.HTTPClient().build_get_query('/hello/world'))
        self.assertEqual(("https://payabbhi.com/hello/world?test=check%23%23", "/hello/world?test=check%23%23"), payabbhi.HTTPClient().build_get_query('/hello/world', params={'test': 'check##'}))
        self.assertEqual(("https://payabbhi.com/hello/world%$?test=check%23%23", "/hello/world%$?&?test=check%23%23"), payabbhi.HTTPClient().build_get_query('/hello/world%$?&', params={'test': 'check##'}))

    def test_build_post_query(self):
        self.assertEqual(('https://payabbhi.com/hello/world', ''), payabbhi.HTTPClient().build_post_query('/hello/world'))
        if sys.version_info[0] <= 2:
            with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
                payabbhi.HTTPClient().build_post_query('/hello/world', data='\xe1')
            self.assertTrue('message: Error in request payload formation' in str(error.exception))
            self.assertTrue('Error in request payload formation' in error.exception.description)
            self.assertIsNone(error.exception.field)

    def test_set_headers(self):
        header = payabbhi.HTTPClient().set_headers(self.client)
        self.assertEqual('application/json', header['Content-Type'])
        header = payabbhi.HTTPClient().set_headers(self.client)
        self.assertEqual('application/json', header['Content-Type'])
        header = payabbhi.HTTPClient().set_headers(self.client)
        self.assertEqual('application/json', header['Content-Type'])

        self.assertTrue('User-Agent' in header)
        self.assertTrue('X-Payabbhi-Client-User-Agent' in header)


    def test_handle_http_method(self):
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().handle_http_method('GETTT', '/hello/world', self.client)
        self.assertTrue('message: Unexpected Method: GETTT, field: method' in str(error.exception))
        self.assertIsNone(error.exception.http_status)
        self.assertTrue('Unexpected Method: GETTT' in error.exception.description)
        self.assertTrue('method' in error.exception.field)

    @responses.activate
    def test_handle_http_code_402(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_payment_collection')
        responses.add(responses.GET, payment_url, status=402,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().request('GET', '/api/v1/payments', self.client)
        self.assertTrue('message: Unexpected HTTP code: 402, http_code: 402' in str(error.exception))
        self.assertEqual(402, error.exception.http_status)
        self.assertTrue('Unexpected HTTP code: 402' in error.exception.description)
        self.assertIsNone(error.exception.field)

        responses.add(responses.POST, payment_url, status=402,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().request('POST', '/api/v1/payments', self.client)
        self.assertTrue('message: Unexpected HTTP code: 402, http_code: 402' in str(error.exception))
        self.assertEqual(402, error.exception.http_status)
        self.assertTrue('Unexpected HTTP code: 402' in error.exception.description)
        self.assertIsNone(error.exception.field)

    @responses.activate
    def test_handle_http_code_400(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_invalid_request')
        count = 'a'
        url = '{0}?count={1}'.format(payment_url, count)
        responses.add(responses.GET, url, status=400,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
            self.client.payment.all(data={'count':'a'})
        self.assertTrue('message: An invalid value was specified for one of the request parameters in the URL, http_code: 400, field: count' in str(error.exception))
        self.assertEqual(400, error.exception.http_status)
        self.assertTrue('An invalid value was specified for one of the request parameters in the URL' in error.exception.description)
        self.assertTrue('count' in error.exception.field)

    @responses.activate
    def test_handle_http_code_404(self):
        result = mock_file('dummy_status_not_found')
        url = payabbhi.api_base + '/api/v1/payments123'
        responses.add(responses.GET, url, status=404,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.InvalidRequestError) as error:
            payabbhi.HTTPClient().request('GET', '/api/v1/payments123', self.client)
        self.assertTrue('message: Request URL /api/v1/payments123 does not exist. Please check documentation, http_code: 404' in str(error.exception))
        self.assertEqual(404, error.exception.http_status)
        self.assertTrue('Request URL /api/v1/payments123 does not exist. Please check documentation' in error.exception.description)

    @responses.activate
    def test_handle_http_code_401(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_authentication')
        responses.add(responses.GET, payment_url, status=401,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.AuthenticationError) as error:
            self.client.payment.all()
        self.assertTrue('message: Incorrect access_id or secret_key provided., http_code: 401' in str(error.exception))
        self.assertEqual(401, error.exception.http_status)
        self.assertTrue('Incorrect access_id or secret_key provided.' in error.exception.description)

    @responses.activate
    def test_handle_http_code_500(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_server_error')
        responses.add(responses.GET, payment_url, status=500,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().request('GET', '/api/v1/payments', self.client)
        self.assertTrue('message: There is some problem with the server, http_code: 500' in str(error.exception))
        self.assertEqual(500, error.exception.http_status)
        self.assertTrue('There is some problem with the server' in error.exception.description)

    @responses.activate
    def test_handle_http_code_502(self):
        result = mock_file('dummy_gateway_error')
        url = '/{0}/{1}/refunds'.format('/api/v1/payments', 'payment_id')
        responses.add(responses.POST, payabbhi.api_base + url, status=502,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.GatewayError) as error:
            payabbhi.HTTPClient().request('POST', url, self.client)
        self.assertTrue('message: Unable to create refund. Please try again., http_code: 502' in str(error.exception))
        self.assertEqual(502, error.exception.http_status)
        self.assertTrue('Unable to create refund. Please try again.' in error.exception.description)

    @responses.activate
    def test_handle_http_code_502_with_empty_message(self):
        result = mock_file('dummy_http_code_502_with_empty_message')
        url = '/{0}/{1}/refunds'.format('/api/v1/payments', 'payment_id')
        responses.add(responses.POST, payabbhi.api_base + url, status=502,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().request('POST', url, self.client)
        self.assertTrue('message: Something did not work as expected on our side, http_code: 502' in str(error.exception))
        self.assertEqual(502, error.exception.http_status)
        self.assertTrue('Something did not work as expected on our side' in error.exception.description)

    @responses.activate
    def test_handle_api_connection(self):
        payabbhi.api_base = 'http://payabbhi.com'
        with self.assertRaises(payabbhi.error.APIConnectionError) as error:
            self.client.payment.all()
        self.assertIsNone(error.exception.field)
        self.assertIsNone(error.exception.http_status)


    @responses.activate
    def test_handle_http_code_500_due_to_json_response(self):
        payment_url = payabbhi.api_base + '/api/v1/payments'
        result = mock_file('dummy_payment_collection')
        result += '}'
        responses.add(responses.GET, payment_url, status=500,
                      body=result, match_querystring=True)
        with self.assertRaises(payabbhi.error.APIError) as error:
            payabbhi.HTTPClient().request('GET', '/api/v1/payments', self.client)
        self.assertTrue('message: Something did not work as expected on our side, http_code: 500' in str(error.exception))
        self.assertEqual(500, error.exception.http_status)
        self.assertTrue('Something did not work as expected on our side' in error.exception.description)



from .api_resource import APIResource


class Subscription(APIResource):

    def __init__(self, client=None):
        super(Subscription, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Subscriptions
        Args:
            data : Dictionary having keys using which subscription list will be filtered
                count:              Count of subscriptions to be retrieved
                skip:               Number of subscriptions to be skipped
                to:                 Subscription list till this timestamp will be retrieved
                from:               Subscription list from this timestamp will be retrieved
                plan_id:            A filter on the subscription list based on the plan_id field
                status:             A filter on the subscription list based on the status field
                billing_method:     A filter on the subscription list based on the billing_method field
                customer_id:        A filter on the subscription list based on the customer_id field
        Returns:
            List of Subscription objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Subscription from given data
        Args:
            data : Dictionary having keys using which subscription has to be created
                plan_id: The unique identifier of the Plan against which this subscription is to be created.
                billing_cycle_count: Total no. of billing cycles. This represents how long the subscription will run.
                customer_id: The unique identifier of the Customer who is subscribing to the plan.
                customer_notification_by: Indicates who is responsible for notification to the customers for important subscription life cycle events.
                trial_duration: Duration of the trial period in days
                quantity: The quantity of the plan, which the customer is subscribing to. Ex : 5 users are subscribing to the plan.
                billing_method: Billing mode of the Subscription. the value can be either recurring or manual.
                due_by_days: No. of days by which the invoices associated with the subscription should be paid starting from the Invoice Issue date. This is applicable only when billing_method is manual.
                notes: key value pair as notes
                upfront_items: List of item objects to be included as upfront charges or set up fees of the subscription.
        Returns:
            Subscription object containing data for created subscription
        """

        return self._post(self.class_url(), data, **kwargs)

    def cancel(self, subscription_id, data=None, **kwargs):
        """"
        Cancels existing Subscription
        Args:
             subscription_id: The identifier of the subscription which needs to be cancelled.
             data : Dictionary having keys using which subscription has to be cancelled
                 at_billing_cycle_end: The flag which determines if the Subscription to be cancelled immediately or at the end of the current billing cycle.
        Returns:
            Subscription object containing data for cancelled subscription
        """
        if data is None:
            data = {}

        url = "{0}/cancel".format(self.instance_url(subscription_id))
        return self._post(url, data, **kwargs)

    def retrieve(self, subscription_id, **kwargs):
        """"
        Retrieve Subscription for given Id
        Args:
            subscription_id : Id for which Subscription object has to be retrieved
        Returns:
            Subscription object for given subscription Id
        """

        return self._retrieve(subscription_id, **kwargs)



from .api_resource import APIResource


class Transfer(APIResource):

    def __init__(self, client=None):
        super(Transfer, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Transfers
        Args:
            data : Dictionary having keys using which transfer list will be filtered
                count:              Count of transfers to be retrieved
                skip:               Number of transfers to be skipped
                to:                 Transfer list till this timestamp will be retrieved
                from:               Transfer list from this timestamp will be retrieved
        Returns:
            List of Transfer objects
        """
        if data is None:
            data = {}
        return super(Transfer, self)._all(data, **kwargs)

    def create(self, source_id, data, **kwargs):
        """"
        Create Transfer from given data
        Args:
            source_id: The identifier of the source for which transfers need to be created.
            data : Dictionary having keys using which transfer has to be created
                transfers: List of transfers to be created with following details
                    recipient_id: The identifier of recipient of this transfer
                    description: Description of the Transfer.
                    amount:  Amount of Transfer
                    currency: Currency used in Transfer
                    notes: key value pair as notes
        Returns:
            Transfer object containing data for created transfers
        """
        url = '/api/v1/payments/' + source_id + '/transfers'
        return self._post(url, data, **kwargs)

    def retrieve(self, transfer_id, **kwargs):
        """"
        Retrieve Transfer for given Id
        Args:
            transfer_id : Id for which Transfer object has to be retrieved
        Returns:
            Transfer object for given transfer Id
        """
        return self._retrieve(transfer_id, **kwargs)



import json
import sys

import payabbhi
import responses
import unittest2

from .helpers import assert_customer, assert_list_of_customers, mock_file


class TestCustomer(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(
            access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.customer_id = 'dummy_customer_id'
        self.customer_url = payabbhi.api_base + '/api/v1/customers'

    @responses.activate
    def test_customer_all(self):
        result = mock_file('dummy_customer_collection')
        responses.add(responses.GET, self.customer_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.customer.all()
        resp = json.loads(result)
        assert_list_of_customers(self, response, resp)

    @responses.activate
    def test_customer_with_options(self):
        result = mock_file('dummy_customer_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(self.customer_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.customer.all(
            data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_customers(self, response, resp)

    @responses.activate
    def test_customer_retrieve(self):
        result = mock_file('dummy_customer')
        url = '{0}/{1}'.format(self.customer_url, self.customer_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.customer.retrieve(self.customer_id)
        resp = json.loads(result)
        assert_customer(self, response, resp)

    @responses.activate
    def test_customer_create(self):
        result = mock_file('dummy_customer')
        url = self.customer_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.customer.create(
            data={'email': 'a@b.com', 'contact_no': '9999999999'})
        resp = json.loads(result)
        assert_customer(self, response, resp)

    @responses.activate
    def test_customer_edit(self):
        result = mock_file('dummy_customer_edit')
        url = '{0}/{1}'.format(self.customer_url, self.customer_id)
        responses.add(responses.PUT, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.customer.edit(
            self.customer_id, data={'email': 'b@c.com', 'contact_no': '1234567890'})
        resp = json.loads(result)
        assert_customer(self, response, resp)



import json
import platform

import payabbhi
import requests

from .error import (APIConnectionError, APIError, AuthenticationError,
                    GatewayError, InvalidRequestError)


class HTTPClient(object):

    def build_get_query(self, path, **options):
        try:
            import urlparse
            from urllib import urlencode
        except ImportError:
            import urllib.parse as urlparse
            from urllib.parse import urlencode

        url = payabbhi.api_base + path

        url_parts = list(urlparse.urlparse(url))
        query = dict(urlparse.parse_qsl(url_parts[4]))

        if 'params' in options:
            query.update(options['params'])

        url_parts[4] = urlencode(query)
        encoded_url = urlparse.urlunparse(url_parts)
        if url_parts[4]:
            path = path + '?' + url_parts[4]
        return encoded_url, path

    def build_post_query(self, path, **options):
        url = payabbhi.api_base + path
        try:
            val = ''
            if 'data' in options:
                val = json.dumps(options['data'])

        except Exception:
            raise InvalidRequestError("Error in request payload formation")
        return url, val

    def _format_app_info(self, client):
        app_info_ua = ""

        app_info = client.get_app_info()

        if app_info:
            if 'name' in app_info:
                app_info_ua += app_info['name']
                if 'version' in app_info:
                    app_info_ua += ('/' + app_info['version'])
                if 'url' in app_info:
                    app_info_ua += (' (' + app_info['url'] + ')')
        return app_info_ua

    def set_headers(self, client):

        app_info = client.get_app_info()

        ua_string = 'Payabbhi/v1 PythonBindings/' + client.VERSION

        user_agent = {
            'bindings_version': client.VERSION,
            'lang': 'python',
            'publisher': 'payabbhi',
            'httplib': requests.__name__
        }

        for attr, func in [['lang_version', platform.python_version],
                           ['platform', platform.platform],
                           ['uname', lambda: ' '.join(platform.uname())]]:
            try:
                val = func()
            except Exception as exception:
                raise APIError(str(exception))
            user_agent[attr] = val

        if app_info:
            user_agent['application'] = app_info
            ua_string += ' ' + self._format_app_info(client)

        headers = {
            'Content-Type': 'application/json',
            'X-Payabbhi-Client-User-Agent':  json.dumps(user_agent),
            'User-Agent': ua_string
        }
        return headers

    def handle_http_method(self, method, path, client, **options):
        if method == 'GET':
            url, path = self.build_get_query(path, **options)
            headers = self.set_headers(client)
            return url, headers, None
        if method == 'POST' or method == 'PUT':
            url, body = self.build_post_query(path, **options)
            headers = self.set_headers(client)
            return url, headers, body
        if method == 'DELETE':
            url = payabbhi.api_base + path
            headers = self.set_headers(client)
            return url, headers, None
        else:
            msg = 'Unexpected Method: ' + method
            raise APIError(msg, None, "method")

    def handle_http_code(self, response):
        if response.status_code == 200:
            return
        else:
            error_response = response.json()
            msg = ''
            field = ''
            if 'error' in error_response:
                msg = error_response['error']['message']
                field = error_response['error']['field']
            if response.status_code == 400 or response.status_code == 404:
                raise InvalidRequestError(msg, response.status_code, field)
            if response.status_code == 401:
                raise AuthenticationError(msg, response.status_code, field)
            elif response.status_code == 500:
                raise APIError(msg, response.status_code)
            elif response.status_code == 502:
                if msg:
                    raise GatewayError(msg, response.status_code)
                else:
                    raise APIError("Something did not work as expected on our side",
                                   response.status_code)
            else:
                msg = "Unexpected HTTP code: " + str(response.status_code)
                raise APIError(msg, response.status_code)

    def request(self, method, path, client, **options):
        url, headers, body = self.handle_http_method(
            method, path, client, **options)
        try:
            response = getattr(requests.Session(), method.lower())(
                url, auth=(client.access_id,
                           client.secret_key),
                headers=headers, data=body)
        except Exception as exception:
            raise APIConnectionError(str(exception))
        try:
            json_resp = response.json()
        except Exception:
            raise APIError(
                "Something did not work as expected on our side", 500)
        self.handle_http_code(response)
        return self.convert_to_object(json_resp, client)

    def convert_to_object(self, resp, client):
        types = {
            'order': payabbhi.resources.Order,
            'payment': payabbhi.resources.Payment,
            'refund': payabbhi.resources.Refund,
            'product': payabbhi.resources.Product,
            'plan': payabbhi.resources.Plan,
            'customer': payabbhi.resources.Customer,
            'subscription': payabbhi.resources.Subscription,
            'invoice': payabbhi.resources.Invoice,
            'invoiceitem': payabbhi.resources.InvoiceItem,
            'event': payabbhi.resources.Event,
            'transfer': payabbhi.resources.Transfer,
            'list': payabbhi.resources.List
        }
        klass_name = resp.get('object')
        if not klass_name:
            return resp
        klass = types.get(klass_name)
        new_object = klass(client=client)
        for key, value in resp.items():
            if isinstance(value, list):
                arr = [self.convert_to_object(i, client) for i in value]
                setattr(new_object, key, arr)
            elif isinstance(value, dict):
                setattr(new_object, key, self.convert_to_object(value, client))
            else:
                setattr(new_object, key, value)
        return new_object



from .api_resource import APIResource


class Invoice(APIResource):

    def __init__(self, client=None):
        super(Invoice, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Invoices
        Args:
            data : Dictionary having keys using which invoice item list will be filtered
                count:              Count of invoices to be retrieved
                skip:               Number of invoices to be skipped
                to:                 Invoice list till this timestamp will be retrieved
                from:               Invoice list from this timestamp will be retrieved
                billing_method:     A filter on the invoice list based on the billing_method field
                due_date_from:      A filter criterion based on the due_date field of the invoice object
                due_date_to:        A filter criterion based on the due_date field of the invoice object
                email:              A filter on the invoice list based on the customer's email field
                subscription_id:    A filter on the invoice list based on the subscription_id field
        Returns:
            List of Invoice objects
        """
        if data is None:
            data = {}
        return self._all(data, **kwargs)

    def create(self, data, **kwargs):
        """"
        Create Invoice Item from given data
        Args:
            data : Dictionary having keys using which invoice item has to be created
                customer_id:  The unique identifier of the Customer who will pay this invoice
                due_date: Latest Timestamp by which the invoice should be paid by the customer
                currency: Currency of the invoice
                invoice_no: Invoice no. at Merchant end which corresponds to this invoice
                line_items: List of invoice item objects to be included in the invoice
                billing_method: Billing method of the invoice
                description: Description of the Invoice
                notify_by: Describes how the customer would be notified about the invoice
                notes: key value pair as notes
                subscription_id: Unique identifier of the subscription for which this invoice will be generated
        Returns:
            Invoice object containing data for created invoice
        """
        return self._post(self.class_url(), data, **kwargs)

    def retrieve(self, invoice_id, **kwargs):
        """"
        Retrieve Invoice for given Id
        Args:
            invoice_id : Id for which Invoice object is to be retrieved
        Returns:
            Invoice object for given invoice id
        """
        return self._retrieve(invoice_id, **kwargs)

    def void(self, invoice_id, data=None, **kwargs):
        """"
        Mark Invoice as Void for a given Id
        Args:
            invoice_id : The unique identifier of the invoice which needs to be voided
        Returns:
            Returns the invoice object if the invoice is voided successfully
        """
        if data is None:
            data = {}

        url = "{0}/void".format(self.instance_url(invoice_id))
        return self._post(url, data, **kwargs)

    def line_items(self, invoice_id, data=None, **kwargs):
        """"
        Retrieve Invoice Items for the Invoice object
        Returns:
            List of Invoice Item objects
        """
        if data is None:
            data = {}

        url = "{0}/line_items".format(self.instance_url(invoice_id))
        return self._get(url, data, **kwargs)

    def payments(self, invoice_id, data=None, **kwargs):
        """"
        Retrieve Payments for the Invoice object
        Returns:
            List of Payment objects
        """
        if data is None:
            data = {}

        url = "{0}/payments".format(self.instance_url(invoice_id))
        return self._get(url, data, **kwargs)



from ..error import InvalidRequestError
from .api_resource import APIResource


class Payment(APIResource):

    def __init__(self, client=None):
        super(Payment, self).__init__(client)

    def all(self, data=None, **kwargs):
        """"
        Get all Payments
        Args:
            data : Dictionary having keys using which payment list will be filtered
                count:           Count of payments to be retrieved
                skip:            Number of payments to be skipped
                to:              Payment list till this timestamp will be retrieved
                from:            Payment list from this timestamp will be retrieved
        Returns:
            List of Payment objects
        """
        if data is None:
            data = {}

        return super(Payment, self)._all(data, **kwargs)

    def retrieve(self, payment_id, **kwargs):
        """"
        Retrieve payment for given Id
        Args:
            payment_id : Id for which Payment object has to be retrieved
        Returns:
            Payment object with given payment Id
        """
        return self._retrieve(payment_id, **kwargs)

    def refunds(self, payment_id, data=None, **kwargs):
        """"
        Retrieve Refunds for given payment Id
        Args:
            payment_id: Payment identifier for which refunds has to be retrieved
            data : Dictionary having keys using which refund list will be filtered
                count:           Count of refunds to be retrieved
                skip:            Number of refunds to be skipped
                to:              Refund list till this timestamp will be retrieved
                from:            Refund list from this timestamp will be retrieved
        Returns:
            Refund list for a payment object
        """
        if data is None:
            data = {}

        url = "{0}/refunds".format(self.instance_url(payment_id))
        return self._get(url, data, **kwargs)

    def capture(self, data=None, **kwargs):
        """"
        Captures the Payment object
        Returns:
            Updated Payment object after getting captured
        """
        if data is None:
            data = {}

        if not hasattr(self, 'id'):
            raise InvalidRequestError('Object Id not set')

        url = self.instance_url(self.id) + '/capture'
        captured_payment = self._post(url, data, **kwargs)
        self.__dict__.update(captured_payment.__dict__)

        return self

    def refund(self, data=None, **kwargs):
        """"
        Refunds Payment object with given data
        Args:
            data : Dictionary having keys using which payment has to be refunded
                amount : Amount for which the payment has to be refunded
                notes: Key value pair as notes
        Returns:
            Refund object that is created
        """
        if data is None:
            data = {}

        if not hasattr(self, 'id'):
            raise InvalidRequestError('Object Id not set')

        url = self.instance_url(self.id) +'/refunds'
        refund = self._post(url, data, **kwargs)

        refunded_payment = self._retrieve(self.id)
        self.__dict__.update(refunded_payment.__dict__)

        return refund

    def transfers(self, payment_id, data=None, **kwargs):
        """"
        Retrieve Transfers for given payment Id
        Args:
            payment_id: Payment identifier for which transfers has to be retrieved
            data : Dictionary having keys using which refund list will be filtered
                count:           Count of transfers to be retrieved
                skip:            Number of transfers to be skipped
                to:              Transfer list till this timestamp will be retrieved
                from:            Transfer list from this timestamp will be retrieved
        Returns:
            Transfer list for a payment object
        """
        if data is None:
            data = {}

        url = "{0}/transfers".format(self.instance_url(payment_id))
        return self._get(url, data, **kwargs)



import json
import sys

import payabbhi
import responses
import unittest2

from .helpers import (assert_list_of_subscriptions, assert_subscription,
                      mock_file)


class TestSubscription(unittest2.TestCase):

    def setUp(self):
        self.client = payabbhi.Client(
            access_id='access_id', secret_key='secret_key')
        payabbhi.api_base = 'https://payabbhi.com'
        self.subscription_id = 'dummy_subscription_id'
        self.subscription_url = payabbhi.api_base + '/api/v1/subscriptions'

    @responses.activate
    def test_subscription_all(self):
        result = mock_file('dummy_subscription_collection')
        responses.add(responses.GET, self.subscription_url, status=200,
                      body=result, match_querystring=True)
        response = self.client.subscription.all()
        resp = json.loads(result)
        assert_list_of_subscriptions(self, response, resp)

    @responses.activate
    def test_subscription_with_options(self):
        result = mock_file('dummy_subscription_collection_filters')
        count = 3
        skip = 2
        url = '{0}?count={1}&skip={2}'.format(
            self.subscription_url, count, skip)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.subscription.all(
            data={'count': count, 'skip': skip})
        resp = json.loads(result)
        assert_list_of_subscriptions(self, response, resp)

    @responses.activate
    def test_subscription_retrieve(self):
        result = mock_file('dummy_subscription')
        url = '{0}/{1}'.format(self.subscription_url, self.subscription_id)
        responses.add(responses.GET, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.subscription.retrieve(self.subscription_id)
        resp = json.loads(result)
        assert_subscription(self, response, resp)

    @responses.activate
    def test_subscription_create(self):
        result = mock_file('dummy_subscription')
        url = self.subscription_url
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.subscription.create(
            data={'plan_id': 'dummy_plan_id', 'customer_id': 'dummy_customer_id', 'billing_cycle_count': 5})
        resp = json.loads(result)
        assert_subscription(self, response, resp)

    @responses.activate
    def test_subscription_cancel(self):
        result = mock_file('dummy_subscription_cancel')
        url = '{0}/{1}/cancel'.format(self.subscription_url,
                                      self.subscription_id)
        responses.add(responses.POST, url, status=200,
                      body=result, match_querystring=True)
        response = self.client.subscription.cancel(self.subscription_id)
        resp = json.loads(result)
        assert_subscription(self, response, resp)



from .payment  import Payment
from .refund   import Refund
from .order   import Order
from .product import Product
from .plan   import Plan
from .customer   import Customer
from .subscription   import Subscription
from .invoice   import Invoice
from .invoice_item   import InvoiceItem
from .event  import Event
from .transfer  import Transfer
from .list   import List
from .api_resource import APIResource

__all__ = [
    'Payment',
    'Refund',
    'Order',
    'Product',
    'Plan',
    'Customer',
    'Subscription',
    'Invoice',
    'InvoiceItem',
    'List',
    'Event',
    'Transfer'
]



from setuptools import setup

with open('LONG_DESCRIPTION.rst') as f:
    long_description = f.read()

setup(
    name="payabbhi",
    version="1.0.3",
    description="Payabbhi Python package provides client library for Payabbhi API's",
    long_description=long_description,
    url="https://github.com/payabbhi/payabbhi-python",
    author="Team Payabbhi",
    author_email="support@payabbhi.com",
    license="MIT",
    install_requires=["requests"],
    package_dir={'payabbhi': 'payabbhi', 'payabbhi.resources': 'payabbhi/resources', 'payabbhi.utility': 'payabbhi/utility'},
    packages=['payabbhi', 'payabbhi.resources', 'payabbhi.test', 'payabbhi.utility'],
    test_suite='payabbhi.test.all',
    tests_require=['unittest2', 'responses'],
    keywords='payabbhi payment processing api checkout india',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ]
)



class PayabbhiError(Exception):

    def __init__(self, description=None, http_status=None,
                 field=None):
        self.description = description
        self.http_status = http_status
        self.field = field
        self._message = self.error_message()
        super(PayabbhiError, self).__init__(self._message)

    def error_message(self):
        msg = "message: " + self.description
        msg = (msg + ", http_code: " + str(self.http_status)) if self.http_status else msg
        msg = (msg + ", field: " + self.field) if self.field else msg
        return msg + "\n"


class APIError(PayabbhiError):
    pass


class APIConnectionError(PayabbhiError):
    pass


class AuthenticationError(PayabbhiError):
    pass


class InvalidRequestError(PayabbhiError):
    pass


class GatewayError(PayabbhiError):
    pass


class SignatureVerificationError(PayabbhiError):
    pass
